# Tracks cpu-stats; docks in WM, if possible
# Author & Copyright: Kero van Gelder, GNU LGPL applies

require 'wise'
require 'kero/unix'

Thread.abort_on_exception = true
List = "ps axo pid,fname,%cpu --sort=-%cpu"

module Wise
  GraphColors = [ Green, Blue, Red ]

  class Graph < Window
    def initialize(parent, sett={})
      @i = 0
      @mem = []
      @hsize, @vsize = sett[:width], sett[:height]
      @hsize.times { |i| @mem[i] = [0, 0, 0, 100] }
      super(parent, sett)
      @hsize, @vsize = width, height

      bind("ConfigureNotify") {
	if width > @hsize
	  (width-@hsize).times {
	    @mem[(@i-1+@mem.size) % @mem.size, 0] = [[0, 0, 0, 100]]
	  }
	  @i += width - @hsize
	  @i %= @mem.size
	end
	@hsize = width
	@vsize = height
      }
    end

    def draw_bar(j, m)
      total = m[0] + m[1] + m[2] + m[3]
      # clear_area(j, 0, 1, @vsize)
      GraphColors[3] = bg
      prev, prev_total = 0, 0
      4.times { |i|
	self.fg = GraphColors[i]
	cur = prev + m[i]
	cur_total = cur * @vsize / total
	draw_line(j, @vsize-prev_total-1, j, @vsize-cur_total)  if m[i] > 0
	prev, prev_total = cur, cur_total
      }
    end

    def draw_content()
      @hsize.times { |j|
	draw_bar(j, @mem[ (@i+@hsize-j-1) % @mem.size ])
      }
    end

    def stats(moment)
      moment[3] = 1  if moment[0] + moment[1] + moment[2] + moment[3] == 0
      @i = (@i-1+@mem.size) % @mem.size
      @mem[@i].replace(moment)
      copy_area(1, 0, @hsize-1, @vsize, 0, 0)
      draw_bar(@hsize-1, @mem[@i])
    end
  end
end

def get_stat_line()
  File.open("/proc/stat", "r") { |f|
    return f.gets.split[1..-1].collect{|v| v.to_i}
  }
end

@win = Wise::Graph.new(nil,
  :title=>"Ruby Cpu Load", :dock=>true, :width=>16, :height=>16,
  :bg=>Wise::Color.new(20, 60, 110)
)

#  @win.bind("Button-Press") { |evt|
#  #  p evt
#  }
@win.bind("Button-Release") { |evt|
  if @w
    @ps_list.text = `#{List}`
  else
    @w = Wise::Window.new(nil, :map=>false, :modal=>@win, :font=>"5x7")
    @ps_list = Wise::Multi.new(@w, :side=>:left, :text=>`#{List}`)
    vlist = Wise::VList.new(@w)
    ["Term", "Hup", "Int", "Kill", "Stop", "Cont"].each { |sig|
      Wise::Button.new(vlist, :text=>sig) {
	# syscall(SysKill, @ps_list.indexed.text.split[0], eval("Sig#{sig}"))
	puts "Sending Sig#{sig} (#{eval(\"Unix::Sig#{sig}\")}) to Pid ???"
      }
    }
    @w.bind("Destroy") { |evt| @w.unmap; evt.no_internals! }
    @t = Thread.new {
      while true
	@ps_list.text = `#{List}`
	sleep 2.5
      end
    }
  end
  @w.map
}

@win.bind("Destroy") { |evt|
  @w.destroy()  if @w
  @t.kill  if @t
  @w = @t = nil
}

@cur = get_stat_line
@thread = Thread.new {
  @top_count = 0
  moment = Array.new(4)
  loop {
    prev = @cur
    @cur = get_stat_line
    4.times { |j|
      moment[j] = @cur[j] - prev[j]
    }
    @win.stats(moment)

    sleep 1
  }
}

Serialize::serialize()
